"""YFinance Sector & Industry Helpers."""

from typing import Dict, List

from yfinance.data import YfData

# https://query1.finance.yahoo.com/v1/finance/sectors/consumer-cyclical?formatted=true&withReturns=true&lang=en-US&region=US&crumb=4RNeMJ56Ymi

# ca_data = yfdata.get_raw_json("https://query1.finance.yahoo.com/v1/finance/sectors/consumer-cyclical?formatted=true&withReturns=true&lang=en-CA&region=CA")

# Historical prices represent market cap in millions.


earnings_cal_url = "https://finance.yahoo.com/calendar/earnings?day=2024-06-28"

econ_cal_url = "https://query1.finance.yahoo.com/ws/screeners/v1/finance/calendar-events?countPerDay=25&economicEventsHighImportanceOnly=true&economicEventsRegionFilter=&endDate=1719309600000&modules=economicEvents&startDate=1719223200000&lang=en-US&region=US"

_QUOTE_SUMMARY_URL_ = "https://query2.finance.yahoo.com/v10/finance/quoteSummary"

# FUTURES_CHAIN_SUMMARY = yfdata.get_raw_json("https://query2.finance.yahoo.com/v10/finance/quoteSummary/ES%3DF", params={"modules": "futuresChain"})


async def get_futures_quotes(symbols):
    # pylint: disable=import-outside-toplevel
    import os  # noqa
    from contextlib import (
        contextmanager,
        redirect_stderr,
        redirect_stdout,
        suppress,
    )  # noqa
    from aiohttp import ClientError  # noqa
    from openbb_yfinance.models.equity_quote import YFinanceEquityQuoteFetcher  # noqa
    from pandas import DataFrame  # noqa

    @contextmanager
    def suppress_all_output():
        with open(os.devnull, "w") as devnull, redirect_stdout(
            devnull
        ), redirect_stderr(devnull):
            yield

    with suppress_all_output(), suppress(ClientError):
        fetcher = YFinanceEquityQuoteFetcher()
        data = await fetcher.fetch_data(
            params={"symbol": ",".join(symbols)}, credentials={}
        )

    df = DataFrame([d.model_dump() for d in data])
    prices = df[["symbol", "bid", "ask", "prev_close"]].copy()
    prices.loc[:, "price"] = round((prices.ask + prices.bid) / 2, 2)
    prices.price = prices.price.fillna(prices.prev_close)
    return prices.set_index("symbol")["price"].to_dict()


def get_futures_symbols(symbol: str):
    """Get the list of futures symbols from the continuation symbol."""
    _symbol = symbol.upper() + "%3DF"
    URL = f"https://query2.finance.yahoo.com/v10/finance/quoteSummary/{_symbol}"
    params = {"modules": "futuresChain"}
    response: Dict = YfData(session=None).get_raw_json(url=URL, params=params)
    futures_symbols: List = []
    if "quoteSummary" in response:
        result = response["quoteSummary"].get("result", [])
        if not result:
            raise ValueError(f"No futures chain found for, {symbol}")
        futures = result[0].get("futuresChain", {})
        if futures:
            futures_symbols = futures.get("futures", [])
    return futures_symbols


SECTOR_SYMBOLS = {
    "energy": "^YH309",
    "basic_materials": "^YH101",
    "industrials": "^YH310",
    "consumer_defensive": "^YH205",
    "consumer_cyclical": "^YH102",
    "financials_services": "^YH103",
    "healthcare": "^YH206",
    "technology": "^YH311",
    "communication_services": "^YH308",
    "utilities": "^YH207",
    "real_estate": "^YH104",
}

# https://query1.finance.yahoo.com/v1/finance/industries/silver?formatted=true&sectorKey=basic-materials&withReturns=true&lang=en-US&region=US&crumb=4RNeMJ56Ymi
# https://finance.yahoo.com/screener/predefined/sec-ind_ind-largest-equities_internet-retail/?offset=0&count=100
# https://query1.finance.yahoo.com/v1/finance/sectors/technology?formatted=false&withReturns=true&lang=en-CA&region=CA&crumb=4RNeMJ56Ymi

INDUSTRY_SYMBOLS = {
    "energy": {
        "oil_and_gas_integrated": "^YH30910030",
        "oil_and_gas_exploration_and_production": "^YH30910020",  # E&P
        "oil_and_gas_midstream": "^YH30910040",
        "oil_and_gas_refining_and_marketing": "^YH30910050",
        "oil_and_gas_equipment_and_services": "^YH30910060",
        "uranium": "^YH30920020",
        "oil_and_gas_drilling": "^YH30910010",
        "thermal_coal": "YH30920010",
    },
    "basic_materials": {
        "specialty_chemicals": "^YH10130020",
        "gold": "^YH10150040",
        "copper": "^YH10150020",
        "building_materials": "^YH10120010",
        "steel": "^YH10160020",
        "agricultural_inputs": "^YH10110010",
        "chemicals": "^YH10130010",
        "other_industrials_metal_and_mining": "^YH10150030",
        "lumber_and_wood_production": "^YH10140010",
        "aluminum": "^YH10150010",
        "other_precious_metals_and_mining": "^YH10150060",
        "coking_coal": "^YH10160010",
        "paper_and_paper_products": "^YH10140020",
        "silver": "^YH10150050",
    },
    "industrials": {
        "aerospace_and_defense": "^YH31010010",
        "specialty_industrial_machinery": "^YH31070020",
        "railroads": "^YH31080030",
        "farm_and_heavy_construction_machinery": "^YH31050010",
        "building_products_and_equipment": "^YH31040030",
        "specialty_business_services": "^YH31020010",
        "integrated_freight_and_logistics": "^YH31080060",
        "waste_management": "^YH31090010",
        "conglomerates": "^YH31030010",
        "industrial_distribution": "^YH31060010",
        "engineering_and_construction": "^YH31040010",
        "staffing_and_employment_services": "^YH31020050",
        "rental_leasing_and_services": "^YH31020030",
        "electrical_equipment_and_parts": "^YH31070060",
        "consulting_services": "^YH31020020",
        "trucking": "^YH31080050",
        "airlines": "^YH31080020",
        "tools_and_accessories": "^YH31070050",
        "security_and_protection_services": "^YH31020040",
        "pollution_and_treatment_controls": "^YH31070040",
        "marine_shipping": "^YH31080040",
        "metal_fabrication": "^YH31070030",
        "infrastructure_operations": "^YH31040020",
        "airports_and_air_services": "^YH31080010",
        "business_equipment_and_supplies": "^YH31070010",
    },
    "consumer_defensive": {
        "discount_stores": "^YH20550010",
        "beverages_non_alcoholic": "^YH20520010",
        "household_and_personal_products": "^YH20525030",
        "packaged_foods": "^YH20525040",
        "tobacco": "^YH20560010",
        "confectioners": "^YH20525010",
        "farm_products": "^YH20525020",
        "beverages_wineries_and_distilleries": "^YH20510020",
        "food_distribution": "^YH20550020",
        "grocery_stores": "^YH20550030",
        "education_and_training_services": "^YH20540010",
        "beverages_brewers": "^YH20510010",
    },
    "consumer_cyclical": {
        "internet_retail": "^YH10280050",
        "auto_manufacturers": "YH10200020",
        "restaurants": "^YH10270010",
        "home_improvement_retail": "^YH10280030",
        "travel_services": "^YH10290050",
        "specialty_retail": "^YH10280060",
        "apparel_retail": "^YH10280010",
        "residential_construction": "^YH10230010",
        "footwear_and_accessories": "^YH10240030",
        "auto_parts": "^YH10200030",
        "packaging_and_containers": "^YH10250010",
        "lodging": "^YH10290030",
        "auto_and_truck_dealerships": "^YH10200010",
        "resorts_and_casinos": "^YH10290040",
        "gambling": "^YH10290010",
        "leisure": "^YH10290020",
        "apparel_manufacturing": "^YH10240020",
        "personal_services": "^YH10260010",
        "furnishings_fixtures_and_appliances": "^YH10220010",
        "recreational_vehicles": "^YH10200040",
        "luxury_goods": "^YH10280040",
        "department_stores": "^YH10280020",
        "textile_manufacturing": "^YH10240010",
    },
    "financial_services": {
        "banks_diversified": "^YH10320010",
        "credit_services": "^YH10360010",
        "asset_management": "^YH10310010",
        "insurance_diversified": "^YH10340060",
        "banks_regional": "^YH10320020",
        "capital_markets": "^YH10330010",
        "financial_data_and_stock_exchanges": "^YH10330020",
        "insurance_property_and_casualty": "^YH10340020",
        "insurance_brokers": "^YH10340050",
        "insurance_life": "^YH10340010",
        "insurance_specialty": "^YH10340040",
        "mortgage_finance": "^YH10320030",
        "insurance_reinsurance": "^YH10340030",
        "shell_companies": "^YH10350010",
        "financial_conglomerates": "^YH10350020",
    },
    "healthcare": {
        "drug_manufacturers_general": "^YH20620010",
        "healthcare_plans": "^YH20630010",
        "medical_devices": "^YH20650010",
        "diagnostic_and_research": "^YH20660010",
        "biotechnology": "^YH20610010",
        "medical_instruments_and_supplies": "^YH20650020",
        "medical_care_facilities": "^YH20645010",
        "drug_manufacturers_specialty_and_generic": "^YH20620020",
        "medical_distribution": "^YH20670010",
        "health_information_services": "^YH20645030",
        "pharmaceutical_retailers": "^YH20645020",
    },
    "technology": {
        "software_infrastructure": "^YH31110030",
        "semiconductors": "^YH31130020",
        "consumer_electronics": "^YH31120030",
        "software_application": "^YH31110020",
        "it_services": "^YH31110010",
        "semiconductor_equipment_and_materials": "^YH31130010",
        "computer_hardware": "^YH31120020",
        "communication_equipment": "^YH31120010",
        "electronic_components": "^YH31120040",
        "scientific_and_technical_instruments": "^YH31120060",
        "solar": "^YH31130030",
        "electronics_and_computer_distribution": "^YH31120050",
    },
    "communication_services": {
        "internet_content_and_information": "^YH30830010",
        "telecom_services": "^YH30810010",
        "entertainment": "^YH30820040",
        "electronic_gaming_and_multimedia": "^YH30830020",
        "advertising_agencies": "^YH30820010",
        "broadcasting": "^YH30820030",
        "publishing": "^YH30820020",
    },
    "utilities": {
        "regulated_electric": "^YH20720020",
        "renewable": "^YH20710020",
        "diversified": "^YH20720040",
        "regulated_gas": "^YH20720030",
        "independent_power_producers": "^YH20710010",
        "regulated_water": "^YH20720010",
    },
    "real_estate": {
        "reit_specialty": "^YH10420080",
        "reit_industrial": "^YH10420030",
        "reit_residential": "^YH10420050",
        "reit_retail": "^YH10420060",
        "reit_healthcare_facilities": "^YH10420010",
        "real_estate_services": "^YH10410020",
        "reit_office": "^YH10420040",
        "reit_diversified": "^YH10420090",
        "reit_mortgage": "^YH10420070",
        "reit_hotel_and_motel": "^YH10420020",
        "real_estate_diversified": "^YH10410030",
        "real_estate_development": "^YH10410010",
    },
}

# with yf.Ticker
# {k: v for k,v  in ticker.basic_info.items()}

# ticker.get_info()
# {'exchange': 'YHD',
# 'quoteType': 'NONE',
# 'symbol': '^YH101',
# 'underlyingSymbol': '^YH101',
# 'shortName': 'Basic Materials (Sector)',
# 'firstTradeDateEpochUtc': 1515594600,
# 'timeZoneFullName': 'America/New_York',
# 'timeZoneShortName': 'EDT',
# 'uuid': '9fe72762-6c2d-39dc-aeca-a3b2d1950883',
# 'messageBoardId': 'finmb_INDEXYH101',
# 'gmtOffSetMilliseconds': -14400000,
# 'maxAge': 86400,
# 'trailingPegRatio': None}



"""YFinance Price Target Model"""

# pylint: disable=unused-argument
import asyncio
from typing import Any, Dict, List, Optional
from warnings import warn

from openbb_core.provider.abstract.fetcher import Fetcher
from openbb_core.provider.standard_models.price_target import (
    PriceTargetData,
    PriceTargetQueryParams,
)
from pydantic import Field
from yfinance import Ticker


class YFinancePriceTargetQueryParams(PriceTargetQueryParams):
    """YFinance Price Target query params."""


class YFinancePriceTargetData(PriceTargetData):
    """YFinance Price Target Data."""

    __alias_dict__ = {
        "published_date": "GradeDate",
        "analyst_company": "Firm",
        "new_grade": "ToGrade",
        "old_grade": "FromGrade",
        "action": "Action",
    }
    new_grade: Optional[str] = Field(
        default=None,
        description="The updated grade.",
        alias="ToGrade",
    )
    old_grade: Optional[str] = Field(
        default=None,
        description="The original grade.",
        alias="FromGrade",
    )
    action: Optional[str] = Field(
        default=None,
        description="The action taken.",
        alias="Action",
    )


class YFinancePriceTargetFetcher(
    Fetcher[YFinancePriceTargetQueryParams, List[YFinancePriceTargetData]]
):
    """YFinance Price Target fetcher."""

    @staticmethod
    async def aextract_data(
        query: YFinancePriceTargetQueryParams,
        credentials: Optional[Dict[str, str]],
        **kwargs: Any,
    ) -> List[Dict]:
        """Extract the raw data from YFinance"""

        symbols = query.symbol.split(",")
        results = []

        async def get_one(symbol):
            """Get the data for one ticker symbol."""
            _ticker = Ticker(symbol)
            ticker = _ticker.get_upgrades_downgrades()
            if not ticker.empty:  # type: ignore
                ticker["symbol"] = symbol
                results.extend(ticker.reset_index().to_dict(orient="records"))  # type: ignore
            else:
                warn(f"Symbol Error: No data found for {symbol}")

        tasks = [get_one(symbol) for symbol in symbols]

        await asyncio.gather(*tasks)

        return results

    @staticmethod
    def transform_data(
        query: YFinancePriceTargetQueryParams,
        data: List[Dict],
        **kwargs: Any,
    ) -> List[YFinancePriceTargetData]:
        """Transform the data."""
        return [YFinancePriceTargetData.model_validate(d) for d in data]



"""YFinance Options Chains Model."""

# pylint: disable=unused-argument

from datetime import datetime
from typing import Any, Dict, List, Optional, Union

from openbb_core.app.model.abstract.error import OpenBBError
from openbb_core.provider.abstract.annotated_result import AnnotatedResult
from openbb_core.provider.abstract.fetcher import Fetcher
from openbb_core.provider.standard_models.options_chains import (
    OptionsChainsData,
    OptionsChainsQueryParams,
)
from openbb_core.provider.utils.errors import EmptyDataError
from pydantic import Field


class YFinanceOptionsChainsQueryParams(OptionsChainsQueryParams):
    """YFinance Options Chains Query Parameters."""


class YFinanceOptionsChainsData(OptionsChainsData):
    """YFinance Options Chains Data."""

    __doc__ = OptionsChainsData.__doc__
    __alias_dict__ = {
        "contract_symbol": "contractSymbol",
        "last_trade_time": "lastTradeDate",
        "last_trade_price": "lastPrice",
        "change_percent": "percentChange",
        "open_interest": "openInterest",
        "implied_volatility": "impliedVolatility",
        "in_the_money": "inTheMoney",
    }

    in_the_money: List[Union[bool, None]] = Field(
        default_factory=list,
        description="Whether the option is in the money.",
    )
    currency: List[Union[str, None]] = Field(
        default_factory=list,
        description="Currency of the option.",
    )


class YFinanceOptionsChainsFetcher(
    Fetcher[YFinanceOptionsChainsQueryParams, YFinanceOptionsChainsData]
):
    """YFinance Options Chains Fetcher."""

    @staticmethod
    def transform_query(params: Dict[str, Any]) -> YFinanceOptionsChainsQueryParams:
        """Transform the query."""
        return YFinanceOptionsChainsQueryParams(**params)

    @staticmethod
    async def aextract_data(
        query: YFinanceOptionsChainsQueryParams,
        credentials: Optional[Dict[str, str]],
        **kwargs: Any,
    ) -> Dict:
        """Extract the raw data from YFinance."""
        # pylint: disable=import-outside-toplevel
        import asyncio  # noqa
        from pandas import concat
        from yfinance import Ticker
        from pytz import timezone

        symbol = query.symbol.upper()
        symbol = "^" + symbol if symbol in ["VIX", "RUT", "SPX", "NDX"] else symbol
        ticker = Ticker(symbol)
        expirations = list(ticker.options)
        if not expirations or len(expirations) == 0:
            raise OpenBBError(f"No options found for {symbol}")
        chains_output: List = []
        underlying = ticker.option_chain(expirations[0])[2]
        underlying_output: Dict = {
            "symbol": symbol,
            "name": underlying.get("longName"),
            "exchange": underlying.get("fullExchangeName"),
            "exchange_tz": underlying.get("exchangeTimezoneName"),
            "currency": underlying.get("currency"),
            "bid": underlying.get("bid"),
            "bid_size": underlying.get("bidSize"),
            "ask": underlying.get("ask"),
            "ask_size": underlying.get("askSize"),
            "last_price": underlying.get(
                "postMarketPrice", underlying.get("regularMarketPrice")
            ),
            "open": underlying.get("regularMarketOpen", None),
            "high": underlying.get("regularMarketDayHigh", None),
            "low": underlying.get("regularMarketDayLow", None),
            "close": underlying.get("regularMarketPrice", None),
            "prev_close": underlying.get("regularMarketPreviousClose", None),
            "change": underlying.get("regularMarketChange", None),
            "change_percent": underlying.get("regularMarketChangePercent", None),
            "volume": underlying.get("regularMarketVolume", None),
            "dividend_yield": float(underlying.get("dividendYield", 0)) / 100,
            "dividend_yield_ttm": underlying.get("trailingAnnualDividendYield", None),
            "year_high": underlying.get("fiftyTwoWeekHigh", None),
            "year_low": underlying.get("fiftyTwoWeekLow", None),
            "ma_50": underlying.get("fiftyDayAverage", None),
            "ma_200": underlying.get("twoHundredDayAverage", None),
            "volume_avg_10d": underlying.get("averageDailyVolume10Day", None),
            "volume_avg_3m": underlying.get("averageDailyVolume3Month", None),
            "market_cap": underlying.get("marketCap", None),
            "shares_outstanding": underlying.get("sharesOutstanding", None),
        }
        tz = timezone(underlying_output.get("exchange_tz", "UTC"))

        underlying_price = underlying_output.get("last_price")

        async def get_chain(ticker, expiration, tz, underlying_price):
            """Get the data for one expiration."""
            exp = datetime.strptime(expiration, "%Y-%m-%d").date()
            now = datetime.now().date()
            dte = (exp - now).days
            calls = ticker.option_chain(expiration, tz=tz)[0]
            calls["option_type"] = "call"
            calls["expiration"] = expiration
            puts = ticker.option_chain(expiration, tz=tz)[1]
            puts["option_type"] = "put"
            puts["expiration"] = expiration
            chain = concat([calls, puts])
            chain = (
                chain.set_index(["strike", "option_type", "contractSymbol"])
                .sort_index()
                .reset_index()
            )
            chain = chain.drop(columns=["contractSize"])
            chain["dte"] = dte
            if underlying_price is not None:
                chain["underlying_price"] = underlying_price
                chain["underlying_symbol"] = symbol
            chain["percentChange"] = chain["percentChange"] / 100

            if len(chain) > 0:
                chains_output.extend(
                    chain.fillna("N/A").replace("N/A", None).to_dict("records")
                )

        await asyncio.gather(
            *[
                get_chain(ticker, expiration, tz, underlying_price)
                for expiration in expirations
            ]
        )

        if not chains_output:
            raise EmptyDataError(f"No data was returned for {symbol}")
        return {"underlying": underlying_output, "chains": chains_output}

    @staticmethod
    def transform_data(
        query: YFinanceOptionsChainsQueryParams,
        data: Dict,
        **kwargs: Any,
    ) -> AnnotatedResult[YFinanceOptionsChainsData]:
        """Transform the data."""
        # pylint: disable=import-outside-toplevel
        from numpy import nan
        from pandas import DataFrame

        if not data:
            raise EmptyDataError()
        metadata = data.get("underlying", {})
        records = data.get("chains", [])
        output = DataFrame(records)
        for col in ["volume", "openInterest"]:
            output[col] = (
                output[col].infer_objects(copy=False).replace({nan: 0}).astype("int64")
            )

        output = output.replace({nan: None})

        return AnnotatedResult(
            result=YFinanceOptionsChainsData.model_validate(output.to_dict("list")),
            metadata=metadata,
        )



"""YFinance Price Target Consensus Model."""

# pylint: disable=unused-argument

from typing import Any, Dict, List, Optional
from warnings import warn

from openbb_core.app.model.abstract.error import OpenBBError
from openbb_core.provider.abstract.fetcher import Fetcher
from openbb_core.provider.standard_models.price_target_consensus import (
    PriceTargetConsensusData,
    PriceTargetConsensusQueryParams,
)
from pydantic import Field, field_validator


class YFinancePriceTargetConsensusQueryParams(PriceTargetConsensusQueryParams):
    """YFinance Price Target Consensus Query."""

    __json_schema_extra__ = {"symbol": {"multiple_items_allowed": True}}

    @field_validator("symbol", mode="before", check_fields=False)
    @classmethod
    def check_symbol(cls, value):
        """Check the symbol."""
        if not value:
            raise OpenBBError("Error: Symbol is a required field for yFinance.")
        return value


class YFinancePriceTargetConsensusData(PriceTargetConsensusData):
    """YFinance Price Target Consensus Data."""

    __alias_dict__ = {
        "target_high": "targetHighPrice",
        "target_low": "targetLowPrice",
        "target_consensus": "targetMeanPrice",
        "target_median": "targetMedianPrice",
        "recommendation": "recommendationKey",
        "recommendation_mean": "recommendationMean",
        "number_of_analysts": "numberOfAnalystOpinions",
        "current_price": "currentPrice",
    }

    recommendation: Optional[str] = Field(
        default=None,
        description="Recommendation - buy, sell, etc.",
    )
    recommendation_mean: Optional[float] = Field(
        default=None,
        description="Mean recommendation score where 1 is strong buy and 5 is strong sell.",
    )
    number_of_analysts: Optional[int] = Field(
        default=None, description="Number of analysts providing opinions."
    )
    current_price: Optional[float] = Field(
        default=None,
        description="Current price of the stock.",
    )
    currency: Optional[str] = Field(
        default=None,
        description="Currency the stock is priced in.",
    )


class YFinancePriceTargetConsensusFetcher(
    Fetcher[
        YFinancePriceTargetConsensusQueryParams, List[YFinancePriceTargetConsensusData]
    ]
):
    """YFinance Price Target Consensus Fetcher."""

    @staticmethod
    def transform_query(
        params: Dict[str, Any]
    ) -> YFinancePriceTargetConsensusQueryParams:
        """Transform the query."""
        return YFinancePriceTargetConsensusQueryParams(**params)

    @staticmethod
    async def aextract_data(
        query: YFinancePriceTargetConsensusQueryParams,
        credentials: Optional[Dict[str, str]],
        **kwargs: Any,
    ) -> List[Dict]:
        """Extract the raw data from YFinance."""
        # pylint: disable=import-outside-toplevel
        import asyncio  # noqa
        from yfinance import Ticker  # noqa

        symbols = query.symbol.split(",")  # type: ignore
        results = []
        fields = [
            "symbol",
            "currentPrice",
            "currency",
            "targetHighPrice",
            "targetLowPrice",
            "targetMeanPrice",
            "targetMedianPrice",
            "recommendationMean",
            "recommendationKey",
            "numberOfAnalystOpinions",
        ]

        async def get_one(symbol):
            """Get the data for one ticker symbol."""
            result = {}
            ticker = {}
            try:
                ticker = Ticker(symbol).get_info()
            except Exception as e:
                warn(f"Error getting data for {symbol}: {e}")
            if ticker:
                for field in fields:
                    if field in ticker:
                        result[field] = ticker.get(field, None)
                if result and result.get("numberOfAnalystOpinions") is not None:
                    results.append(result)

        tasks = [get_one(symbol) for symbol in symbols]

        await asyncio.gather(*tasks)

        return results

    @staticmethod
    def transform_data(
        query: YFinancePriceTargetConsensusQueryParams,
        data: List[Dict],
        **kwargs: Any,
    ) -> List[YFinancePriceTargetConsensusData]:
        """Transform the data."""
        return [YFinancePriceTargetConsensusData.model_validate(d) for d in data]



"""Yahoo Finance provider module."""

from openbb_core.provider.abstract.provider import Provider
from openbb_yfinance.models.active import YFActiveFetcher
from openbb_yfinance.models.aggressive_small_caps import YFAggressiveSmallCapsFetcher
from openbb_yfinance.models.available_indices import YFinanceAvailableIndicesFetcher
from openbb_yfinance.models.balance_sheet import YFinanceBalanceSheetFetcher
from openbb_yfinance.models.cash_flow import YFinanceCashFlowStatementFetcher
from openbb_yfinance.models.company_news import YFinanceCompanyNewsFetcher
from openbb_yfinance.models.crypto_historical import YFinanceCryptoHistoricalFetcher
from openbb_yfinance.models.currency_historical import YFinanceCurrencyHistoricalFetcher
from openbb_yfinance.models.equity_historical import YFinanceEquityHistoricalFetcher
from openbb_yfinance.models.equity_profile import YFinanceEquityProfileFetcher
from openbb_yfinance.models.equity_quote import YFinanceEquityQuoteFetcher
from openbb_yfinance.models.etf_info import YFinanceEtfInfoFetcher
from openbb_yfinance.models.futures_curve import YFinanceFuturesCurveFetcher
from openbb_yfinance.models.futures_historical import YFinanceFuturesHistoricalFetcher
from openbb_yfinance.models.gainers import YFGainersFetcher
from openbb_yfinance.models.growth_tech_equities import YFGrowthTechEquitiesFetcher
from openbb_yfinance.models.historical_dividends import (
    YFinanceHistoricalDividendsFetcher,
)
from openbb_yfinance.models.income_statement import YFinanceIncomeStatementFetcher
from openbb_yfinance.models.index_historical import (
    YFinanceIndexHistoricalFetcher,
)
from openbb_yfinance.models.key_executives import YFinanceKeyExecutivesFetcher
from openbb_yfinance.models.key_metrics import YFinanceKeyMetricsFetcher
from openbb_yfinance.models.losers import YFLosersFetcher
from openbb_yfinance.models.options_chains import YFinanceOptionsChainsFetcher
from openbb_yfinance.models.price_target_consensus import (
    YFinancePriceTargetConsensusFetcher,
)
from openbb_yfinance.models.share_statistics import YFinanceShareStatisticsFetcher
from openbb_yfinance.models.undervalued_growth_equities import (
    YFUndervaluedGrowthEquitiesFetcher,
)
from openbb_yfinance.models.undervalued_large_caps import YFUndervaluedLargeCapsFetcher

yfinance_provider = Provider(
    name="yfinance",
    website="https://finance.yahoo.com",
    description="""Yahoo! Finance is a web-based platform that offers financial news,
data, and tools for investors and individuals interested in tracking and analyzing
financial markets and assets.""",
    fetcher_dict={
        "AvailableIndices": YFinanceAvailableIndicesFetcher,
        "BalanceSheet": YFinanceBalanceSheetFetcher,
        "CashFlowStatement": YFinanceCashFlowStatementFetcher,
        "CompanyNews": YFinanceCompanyNewsFetcher,
        "CryptoHistorical": YFinanceCryptoHistoricalFetcher,
        "CurrencyHistorical": YFinanceCurrencyHistoricalFetcher,
        "EquityActive": YFActiveFetcher,
        "EquityAggressiveSmallCaps": YFAggressiveSmallCapsFetcher,
        "EquityGainers": YFGainersFetcher,
        "EquityHistorical": YFinanceEquityHistoricalFetcher,
        "EquityInfo": YFinanceEquityProfileFetcher,
        "EquityLosers": YFLosersFetcher,
        "EquityQuote": YFinanceEquityQuoteFetcher,
        "EquityUndervaluedGrowth": YFUndervaluedGrowthEquitiesFetcher,
        "EquityUndervaluedLargeCaps": YFUndervaluedLargeCapsFetcher,
        "EtfHistorical": YFinanceEquityHistoricalFetcher,
        "EtfInfo": YFinanceEtfInfoFetcher,
        "FuturesCurve": YFinanceFuturesCurveFetcher,
        "FuturesHistorical": YFinanceFuturesHistoricalFetcher,
        "GrowthTechEquities": YFGrowthTechEquitiesFetcher,
        "HistoricalDividends": YFinanceHistoricalDividendsFetcher,
        "IncomeStatement": YFinanceIncomeStatementFetcher,
        "IndexHistorical": YFinanceIndexHistoricalFetcher,
        "KeyExecutives": YFinanceKeyExecutivesFetcher,
        "KeyMetrics": YFinanceKeyMetricsFetcher,
        "MarketIndices": YFinanceIndexHistoricalFetcher,
        "OptionsChains": YFinanceOptionsChainsFetcher,
        "PriceTargetConsensus": YFinancePriceTargetConsensusFetcher,
        "ShareStatistics": YFinanceShareStatisticsFetcher,
    },
    repr_name="Yahoo Finance",
)
